@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1997,2007,2016 Andrew G Morgan <morgan@kernel.org>
  *
- * This file deals with setting capabilities on files.
+ * This file deals with getting/setting capabilities from/on files.
  */
 
 #ifndef _DEFAULT_SOURCE
@@ -45,7 +45,7 @@ extern int fremovexattr(int, const char *);
 
 #include "libcap.h"
 
-#if defined (VFS_CAP_U32) && defined (XATTR_NAME_CAPS)
+#ifdef VFS_CAP_U32
 
 #if VFS_CAP_U32 != __CAP_BLKS
 # error VFS representation of capabilities is not the same size as kernel
